@@ -1,6 +1,6 @@
 /* Miscellaneous error functions
 
-   Copyright (C) 2005 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 #include <system.h>
 #include <paxlib.h>
 
-int exit_status;
+int exit_status = PAXEXIT_SUCCESS;
 
 void
 pax_exit ()
