@@ -1,7 +1,7 @@
 /* Common declarations for the tar program.
 
    Copyright (C) 1988, 1992, 1993, 1994, 1996, 1997, 1999, 2000, 2001,
-   2003, 2004 Free Software Foundation, Inc.
+   2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -15,7 +15,7 @@
 
    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 /* Declare the GNU tar archive format.  */
 #include "tar.h"
@@ -29,10 +29,6 @@
 #define UNAME_FIELD_SIZE   32
 #define GNAME_FIELD_SIZE   32
 
-/* FIXME */
-#define MAXOCTAL11      017777777777L
-#define MAXOCTAL7       07777777
-
 
 
 /* Some various global definitions.  */
@@ -47,31 +43,10 @@
 # define GLOBAL extern
 #endif
 
-/* Exit status for GNU tar.  Let's try to keep this list as simple as
-   possible.  -d option strongly invites a status different for unequal
-   comparison and other errors.  */
-GLOBAL int exit_status;
+#define TAREXIT_SUCCESS PAXEXIT_SUCCESS
+#define TAREXIT_DIFFERS PAXEXIT_DIFFERS
+#define TAREXIT_FAILURE PAXEXIT_FAILURE
 
-#define TAREXIT_SUCCESS 0
-#define TAREXIT_DIFFERS 1
-#define TAREXIT_FAILURE 2
-
-/* Both WARN and ERROR write a message on stderr and continue processing,
-   however ERROR manages so tar will exit unsuccessfully.  FATAL_ERROR
-   writes a message on stderr and aborts immediately, with another message
-   line telling so.  USAGE_ERROR works like FATAL_ERROR except that the
-   other message line suggests trying --help.  All four macros accept a
-   single argument of the form ((0, errno, _("FORMAT"), Args...)).  errno
-   is zero when the error is not being detected by the system.  */
-
-#define WARN(Args) \
-  error Args
-#define ERROR(Args) \
-  (error Args, exit_status = TAREXIT_FAILURE)
-#define FATAL_ERROR(Args) \
-  (error Args, fatal_exit ())
-#define USAGE_ERROR(Args) \
-  (error Args, usage (TAREXIT_FAILURE))
 
 #include "arith.h"
 #include <backupfile.h>
@@ -80,7 +55,13 @@ GLOBAL int exit_status;
 #include <modechange.h>
 #include <quote.h>
 #include <safe-read.h>
+#include <stat-time.h>
 #include <timespec.h>
+#define obstack_chunk_alloc xmalloc
+#define obstack_chunk_free free
+#include <obstack.h>
+
+#include <paxlib.h>
 
 /* Log base 2 of common values.  */
 #define LG_8 3
@@ -133,7 +114,13 @@ GLOBAL bool utc_option;
    than newer_mtime_option.  */
 GLOBAL int after_date_option;
 
-GLOBAL bool atime_preserve_option;
+enum atime_preserve
+{
+  no_atime_preserve,
+  replace_atime_preserve,
+  system_atime_preserve
+};
+GLOBAL enum atime_preserve atime_preserve_option;
 
 GLOBAL bool backup_option;
 
@@ -142,7 +129,15 @@ GLOBAL enum backup_type backup_type;
 
 GLOBAL bool block_number_option;
 
-GLOBAL bool checkpoint_option;
+GLOBAL unsigned checkpoint_option;
+
+enum checkpoint_style
+  {
+    checkpoint_text,
+    checkpoint_dot
+  };
+
+GLOBAL enum checkpoint_style checkpoint_style;
 
 /* Specified name of compression program, or "gzip" as implied by -z.  */
 GLOBAL const char *use_compress_program_option;
@@ -155,11 +150,18 @@ GLOBAL int check_links_option;
 /* Patterns that match file names to be excluded.  */
 GLOBAL struct exclude *excluded;
 
-/* Exclude directories containing a cache directory tag. */
-GLOBAL bool exclude_caches_option;
-
-/* Specified file containing names to work on.  */
-GLOBAL const char *files_from_option;
+enum exclusion_tag_type
+  {
+    exclusion_tag_none,
+     /* Exclude the directory contents, but preserve the directory
+	itself and the exclusion tag file */
+    exclusion_tag_contents,
+    /* Exclude everything below the directory, preserving the directory
+       itself */
+    exclusion_tag_under,
+    /* Exclude entire directory  */
+    exclusion_tag_all,
+  };
 
 /* Specified value to be put into tar file in place of stat () results, or
    just -1 if such an override should not take place.  */
@@ -196,25 +198,31 @@ GLOBAL const char *listed_incremental_option;
 /* Specified mode change string.  */
 GLOBAL struct mode_change *mode_option;
 
-GLOBAL bool multi_volume_option;
+/* Initial umask, if needed for mode change string.  */
+GLOBAL mode_t initial_umask;
 
-/* The same variable holds the time, whether mtime or ctime.  Just fake a
-   non-existing option, for making the code clearer, elsewhere.  */
-#define newer_ctime_option newer_mtime_option
+GLOBAL bool multi_volume_option;
 
 /* Specified threshold date and time.  Files having an older time stamp
    do not get archived (also see after_date_option above).  */
 GLOBAL struct timespec newer_mtime_option;
 
+/* If true, override actual mtime (see below) */
+GLOBAL bool set_mtime_option;
+/* Value to be put in mtime header field instead of the actual mtime */
+GLOBAL struct timespec mtime_option;
+
 /* Return true if newer_mtime_option is initialized.  */
 #define NEWER_OPTION_INITIALIZED(opt) (0 <= (opt).tv_nsec)
 
 /* Return true if the struct stat ST's M time is less than
    newer_mtime_option.  */
 #define OLDER_STAT_TIME(st, m) \
-  ((st).st_##m##time < newer_mtime_option.tv_sec \
-   || ((st).st_##m##time == newer_mtime_option.tv_sec \
-       && TIMESPEC_NS ((st).st_##m##tim) < newer_mtime_option.tv_nsec))
+  (timespec_cmp (get_stat_##m##time (&(st)), newer_mtime_option) < 0)
+
+/* Likewise, for struct tar_stat_info ST.  */
+#define OLDER_TAR_STAT_TIME(st, m) \
+  (timespec_cmp ((st).m##time, newer_mtime_option) < 0)
 
 /* Zero if there is no recursion, otherwise FNM_LEADING_DIR.  */
 GLOBAL int recursion_option;
@@ -254,6 +262,8 @@ GLOBAL size_t strip_name_components;
 GLOBAL bool show_omitted_dirs_option;
 
 GLOBAL bool sparse_option;
+GLOBAL unsigned tar_sparse_major;
+GLOBAL unsigned tar_sparse_minor;
 
 GLOBAL bool starting_file_option;
 
@@ -266,6 +276,15 @@ GLOBAL bool totals_option;
 
 GLOBAL bool touch_option;
 
+GLOBAL char *to_command_option;
+GLOBAL bool ignore_command_error_option;
+
+/* Restrict some potentially harmful tar options */
+GLOBAL bool restrict_option;
+
+/* Return true if the extracted files are not being written to disk */
+#define EXTRACT_OVER_PIPE (to_stdout_option || to_command_option)
+
 /* Count how many times the option has been set, multiple setting yields
    more verbose behavior.  Value 0 means no verbosity, 1 means file name
    only, 2 means file name and all attributes.  More than 2 is just like 2.  */
@@ -287,21 +306,20 @@ GLOBAL int archive;
 /* Nonzero when outputting to /dev/null.  */
 GLOBAL bool dev_null_output;
 
-/* Timestamp for when we started execution.  */
-#if HAVE_CLOCK_GETTIME
-  GLOBAL struct timespec start_timespec;
-# define start_time (start_timespec.tv_sec)
-#else
-  GLOBAL time_t start_time;
-#endif
+/* Timestamps: */
+GLOBAL struct timespec start_time;        /* when we started execution */
+GLOBAL struct timespec volume_start_time; /* when the current volume was
+					     opened*/
+GLOBAL struct timespec last_stat_time;    /* when the statistics was last
+					     computed */
 
 GLOBAL struct tar_stat_info current_stat_info;
 
 /* List of tape drive names, number of such tape drives, allocated number,
    and current cursor in list.  */
 GLOBAL const char **archive_name_array;
-GLOBAL int archive_names;
-GLOBAL int allocated_archive_names;
+GLOBAL size_t archive_names;
+GLOBAL size_t allocated_archive_names;
 GLOBAL const char **archive_name_cursor;
 
 /* Output index file name.  */
@@ -310,16 +328,15 @@ GLOBAL char const *index_file_name;
 /* Structure for keeping track of filenames and lists thereof.  */
 struct name
   {
-    struct name *next;
-    size_t length;		/* cached strlen(name) */
+    struct name *next;          /* Link to the next element */
+    int change_dir;		/* Number of the directory to change to.
+				   Set with the -C option. */
     uintmax_t found_count;	/* number of times a matching file has
 				   been found */
-    int isdir;
-    char firstch;		/* first char is literally matched */
-    char regexp;		/* this name is a regexp, not literal */
-    int change_dir;		/* set with the -C option */
+    int matching_flags;		/* this name is a regexp, not literal */
     char const *dir_contents;	/* for incremental_option */
-    char fake;			/* dummy entry */
+
+    size_t length;		/* cached strlen(name) */
     char name[1];
   };
 
@@ -331,6 +348,25 @@ GLOBAL bool seekable_archive;
 
 GLOBAL dev_t root_device;
 
+/* Unquote filenames */
+GLOBAL bool unquote_option;
+
+GLOBAL bool test_label_option; /* Test archive volume label and exit */
+
+/* Show file or archive names after transformation.
+   In particular, when creating archive in verbose mode, list member names
+   as stored in the archive */
+GLOBAL bool show_transformed_names_option;
+
+/* Delay setting modification times and permissions of extracted directories
+   until the end of extraction. This variable helps correctly restore directory
+   timestamps from archives with an unusual member order. It is automatically
+   set for incremental archives. */
+GLOBAL bool delay_directory_restore_option;
+
+/* Warn about implicit use of the wildcards in command line arguments.
+   (Default for tar prior to 1.15.91, but changed afterwards */
+GLOBAL bool warn_regex_usage;
 
 /* Declarations for each module.  */
 
@@ -348,31 +384,40 @@ extern enum access_mode access_mode;
 /* Module buffer.c.  */
 
 extern FILE *stdlis;
-extern char *save_name;
-extern off_t save_sizeleft;
-extern off_t save_totsize;
 extern bool write_archive_to_stdout;
+extern char *volume_label;
+extern char *continued_file_name;
+extern uintmax_t continued_file_size;
+extern uintmax_t continued_file_offset;
 
-size_t available_space_after (union block *);
+size_t available_space_after (union block *pointer);
 off_t current_block_ordinal (void);
 void close_archive (void);
 void closeout_volume_number (void);
+void compute_duration (void);
 union block *find_next_block (void);
 void flush_read (void);
 void flush_write (void);
 void flush_archive (void);
 void init_volume_number (void);
-void open_archive (enum access_mode);
-void print_total_written (void);
+void open_archive (enum access_mode mode);
+void print_total_stats (void);
 void reset_eof (void);
-void set_next_block_after (union block *);
+void set_next_block_after (union block *block);
 void clear_read_error_count (void);
 void xclose (int fd);
-void archive_write_error (ssize_t) __attribute__ ((noreturn));
+void archive_write_error (ssize_t status) __attribute__ ((noreturn));
 void archive_read_error (void);
 off_t seek_archive (off_t size);
 void set_start_time (void);
 
+void mv_begin (struct tar_stat_info *st);
+void mv_end (void);
+void mv_total_size (off_t size);
+void mv_size_left (off_t size);
+
+void buffer_write_global_xheader (void);
+
 /* Module create.c.  */
 
 enum dump_status
@@ -383,13 +428,20 @@ enum dump_status
     dump_status_not_implemented
   };
 
-bool file_dumpable_p (struct tar_stat_info *);
+void add_exclusion_tag (const char *name, enum exclusion_tag_type type,
+			bool (*)(const char*));
+bool cachedir_file_p (const char *name);
+
+bool file_dumpable_p (struct tar_stat_info *st);
 void create_archive (void);
 void pad_archive (off_t size_left);
-void dump_file (char *, int, dev_t);
+void dump_file (const char *st, int top_level, dev_t parent_device);
 union block *start_header (struct tar_stat_info *st);
-void finish_header (struct tar_stat_info *, union block *, off_t);
+void finish_header (struct tar_stat_info *st, union block *header,
+		    off_t block_ordinal);
 void simple_finish_header (union block *header);
+union block * write_extended (bool global, struct tar_stat_info *st,
+			      union block *old_header);
 union block *start_private_header (const char *name, size_t size);
 void write_eot (void);
 void check_links (void);
@@ -406,16 +458,16 @@ void check_links (void);
 #define UNAME_TO_CHARS(name,buf) string_to_chars (name, buf, sizeof(buf))
 #define GNAME_TO_CHARS(name,buf) string_to_chars (name, buf, sizeof(buf))
 
-void gid_to_chars (gid_t, char *, size_t);
-void major_to_chars (major_t, char *, size_t);
-void minor_to_chars (minor_t, char *, size_t);
-void mode_to_chars (mode_t, char *, size_t);
-void off_to_chars (off_t, char *, size_t);
-void size_to_chars (size_t, char *, size_t);
-void time_to_chars (time_t, char *, size_t);
-void uid_to_chars (uid_t, char *, size_t);
-void uintmax_to_chars (uintmax_t, char *, size_t);
-void string_to_chars (char *, char *, size_t);
+bool gid_to_chars (gid_t gid, char *buf, size_t size);
+bool major_to_chars (major_t m, char *buf, size_t size);
+bool minor_to_chars (minor_t m, char *buf, size_t size);
+bool mode_to_chars (mode_t m, char *buf, size_t size);
+bool off_to_chars (off_t off, char *buf, size_t size);
+bool size_to_chars (size_t v, char *buf, size_t size);
+bool time_to_chars (time_t t, char *buf, size_t size);
+bool uid_to_chars (uid_t uid, char *buf, size_t size);
+bool uintmax_to_chars (uintmax_t v, char *buf, size_t size);
+void string_to_chars (char const *s, char *buf, size_t size);
 
 /* Module diffarch.c.  */
 
@@ -427,11 +479,10 @@ void verify_volume (void);
 
 /* Module extract.c.  */
 
-extern bool we_are_root;
 void extr_init (void);
 void extract_archive (void);
 void extract_finish (void);
-void fatal_exit (void) __attribute__ ((noreturn));
+bool rename_directory (char *src, char *dst);
 
 /* Module delete.c.  */
 
@@ -439,10 +490,16 @@ void delete_archive_members (void);
 
 /* Module incremen.c.  */
 
-char *get_directory_contents (char *, dev_t);
+char *get_directory_contents (char *dir_name, dev_t device);
+const char *append_incremental_renames (const char *dump);
 void read_directory_file (void);
 void write_directory_file (void);
-void purge_directory (char const *);
+void purge_directory (char const *directory_name);
+void list_dumpdir (char *buffer, size_t size);
+void update_parent_directory (const char *name);
+
+size_t dumpdir_size (const char *p);
+bool is_dumpdir (struct tar_stat_info *stat_info);
 
 /* Module list.c.  */
 
@@ -456,25 +513,14 @@ enum read_header
   HEADER_FAILURE		/* ill-formed header, or bad checksum */
 };
 
-struct xheader
-{
-  struct obstack *stk;
-  size_t size;
-  char *buffer;
-};
-
-GLOBAL struct xheader extended_header;
 extern union block *current_header;
 extern enum archive_format current_format;
 extern size_t recent_long_name_blocks;
 extern size_t recent_long_link_blocks;
 
-void decode_header (union block *, struct tar_stat_info *,
-		    enum archive_format *, int);
-#define STRINGIFY_BIGINT(i, b) \
-  stringify_uintmax_t_backwards ((uintmax_t) (i), (b) + UINTMAX_STRSIZE_BOUND)
-char *stringify_uintmax_t_backwards (uintmax_t, char *);
-char const *tartime (time_t);
+void decode_header (union block *header, struct tar_stat_info *stat_info,
+		    enum archive_format *format_pointer, int do_user_group);
+char const *tartime (struct timespec t, bool full_time);
 
 #define GID_FROM_HEADER(where) gid_from_header (where, sizeof (where))
 #define MAJOR_FROM_HEADER(where) major_from_header (where, sizeof (where))
@@ -486,36 +532,38 @@ char const *tartime (time_t);
 #define UID_FROM_HEADER(where) uid_from_header (where, sizeof (where))
 #define UINTMAX_FROM_HEADER(where) uintmax_from_header (where, sizeof (where))
 
-gid_t gid_from_header (const char *, size_t);
-major_t major_from_header (const char *, size_t);
-minor_t minor_from_header (const char *, size_t);
-mode_t mode_from_header (const char *, size_t);
-off_t off_from_header (const char *, size_t);
-size_t size_from_header (const char *, size_t);
-time_t time_from_header (const char *, size_t);
-uid_t uid_from_header (const char *, size_t);
-uintmax_t uintmax_from_header (const char *, size_t);
+gid_t gid_from_header (const char *buf, size_t size);
+major_t major_from_header (const char *buf, size_t size);
+minor_t minor_from_header (const char *buf, size_t size);
+mode_t mode_from_header (const char *buf, size_t size);
+off_t off_from_header (const char *buf, size_t size);
+size_t size_from_header (const char *buf, size_t size);
+time_t time_from_header (const char *buf, size_t size);
+uid_t uid_from_header (const char *buf, size_t size);
+uintmax_t uintmax_from_header (const char * buf, size_t size);
 
 void list_archive (void);
-void print_for_mkdir (char *, int, mode_t);
-void print_header (struct tar_stat_info *, off_t);
-void read_and (void (*) (void));
-enum read_header read_header (bool);
+void print_for_mkdir (char *dirname, int length, mode_t mode);
+void print_header (struct tar_stat_info *st, off_t block_ordinal);
+void read_and (void (*do_something) (void));
+enum read_header read_header_primitive (bool raw_extended_headers,
+					struct tar_stat_info *info);
+enum read_header read_header (bool raw_extended_headers);
 enum read_header tar_checksum (union block *header, bool silent);
-void skip_file (off_t);
+void skip_file (off_t size);
 void skip_member (void);
 
-/* Module mangle.c.  */
-
-void extract_mangle (void);
-
 /* Module misc.c.  */
 
-void assign_string (char **, const char *);
-char *quote_copy_string (const char *);
-int unquote_string (char *);
+void assign_string (char **dest, const char *src);
+char *quote_copy_string (const char *str);
+int unquote_string (char *str);
 
-size_t dot_dot_prefix_len (char const *);
+void code_ns_fraction (int ns, char *p);
+char const *code_timespec (struct timespec ts, char *sbuf);
+enum { BILLION = 1000000000, LOG10_BILLION = 9 };
+enum { TIMESPEC_STRSIZE_BOUND =
+         UINTMAX_STRSIZE_BOUND + LOG10_BILLION + sizeof "-." - 1 };
 
 enum remove_option
 {
@@ -529,108 +577,69 @@ enum remove_option
      (RECURSIVE_REMOVE_OPTION value), which removes them in
      prepare_to_extract() phase. However, with the addition of more
      meta-info to the incremental dumps, this should become unnecessary */
-  WANT_DIRECTORY_REMOVE_OPTION  
+  WANT_DIRECTORY_REMOVE_OPTION
 };
-int remove_any_file (const char *, enum remove_option);
-bool maybe_backup_file (const char *, int);
+int remove_any_file (const char *file_name, enum remove_option option);
+bool maybe_backup_file (const char *file_name, bool this_is_the_archive);
 void undo_last_backup (void);
 
-int deref_stat (bool, char const *, struct stat *);
+int deref_stat (bool deref, char const *name, struct stat *buf);
 
-int chdir_arg (char const *);
-void chdir_do (int);
+int chdir_arg (char const *dir);
+void chdir_do (int dir);
 
-void decode_mode (mode_t, char *);
-
-void chdir_fatal (char const *) __attribute__ ((noreturn));
-void chmod_error_details (char const *, mode_t);
-void chown_error_details (char const *, uid_t, gid_t);
-void close_error (char const *);
-void close_warn (char const *);
 void close_diag (char const *name);
-void exec_fatal (char const *) __attribute__ ((noreturn));
-void link_error (char const *, char const *);
-void mkdir_error (char const *);
-void mkfifo_error (char const *);
-void mknod_error (char const *);
-void open_error (char const *);
-void open_fatal (char const *) __attribute__ ((noreturn));
-void open_warn (char const *);
 void open_diag (char const *name);
-void read_error (char const *);
-void read_error_details (char const *, off_t, size_t);
-void read_fatal (char const *) __attribute__ ((noreturn));
-void read_fatal_details (char const *, off_t, size_t) __attribute__ ((noreturn));
-void read_warn_details (char const *, off_t, size_t);
 void read_diag_details (char const *name, off_t offset, size_t size);
-void readlink_error (char const *);
-void readlink_warn (char const *);
 void readlink_diag (char const *name);
-void savedir_error (char const *);
-void savedir_warn (char const *);
 void savedir_diag (char const *name);
-void seek_error (char const *);
-void seek_error_details (char const *, off_t);
-void seek_warn (char const *);
-void seek_warn_details (char const *, off_t);
-void seek_diag_details (char const *, off_t);
-void stat_error (char const *);
-void stat_warn (char const *);
+void seek_diag_details (char const *name, off_t offset);
 void stat_diag (char const *name);
-void symlink_error (char const *, char const *);
-void truncate_error (char const *);
-void truncate_warn (char const *);
-void unlink_error (char const *);
-void utime_error (char const *);
-void waitpid_error (char const *);
-void write_error (char const *);
-void write_error_details (char const *, size_t, size_t);
-void write_fatal (char const *) __attribute__ ((noreturn));
-void write_fatal_details (char const *, ssize_t, size_t)
+void write_error_details (char const *name, size_t status, size_t size);
+void write_fatal (char const *name) __attribute__ ((noreturn));
+void write_fatal_details (char const *name, ssize_t status, size_t size)
      __attribute__ ((noreturn));
 
 pid_t xfork (void);
-void xpipe (int[2]);
+void xpipe (int fd[2]);
 
-void *page_aligned_alloc (void **, size_t);
+void *page_aligned_alloc (void **ptr, size_t size);
+int set_file_atime (int fd, char const *file,
+		    struct timespec const timespec[2]);
 
 /* Module names.c.  */
 
 extern struct name *gnu_list_name;
 
-void gid_to_gname (gid_t, char **gname);
-int gname_to_gid (char const *, gid_t *);
-void uid_to_uname (uid_t, char **uname);
-int uname_to_uid (char const *, uid_t *);
+void gid_to_gname (gid_t gid, char **gname);
+int gname_to_gid (char const *gname, gid_t *pgid);
+void uid_to_uname (uid_t uid, char **uname);
+int uname_to_uid (char const *uname, uid_t *puid);
 
-void init_names (void);
-void name_add (const char *);
 void name_init (void);
+void name_add_name (const char *name, int matching_flags);
+void name_add_dir (const char *name);
 void name_term (void);
-char *name_next (int);
-void name_close (void);
+const char *name_next (int change_dirs);
 void name_gather (void);
-struct name *addname (char const *, int);
-int name_match (const char *);
+struct name *addname (char const *string, int change_dir);
+bool name_match (const char *name);
 void names_notfound (void);
 void collect_and_sort_names (void);
-struct name *name_scan (const char *);
+struct name *name_scan (const char *name);
 char *name_from_list (void);
 void blank_name_list (void);
-char *new_name (const char *, const char *);
-char *safer_name_suffix (char const *, bool);
+char *new_name (const char *dir_name, const char *name);
 size_t stripped_prefix_len (char const *file_name, size_t num);
-bool all_names_found (struct tar_stat_info *);
+bool all_names_found (struct tar_stat_info *st);
 
-bool excluded_name (char const *);
+bool excluded_name (char const *name);
 
-void add_avoided_name (char const *);
-bool is_avoided_name (char const *);
-bool is_individual_file (char const *);
+void add_avoided_name (char const *name);
+bool is_avoided_name (char const *name);
+bool is_individual_file (char const *name);
 
-bool contains_dot_dot (char const *);
-
-bool removed_prefixes_p (void);
+bool contains_dot_dot (char const *name);
 
 #define ISFOUND(c) ((occurrence_option == 0) ? (c)->found_count : \
                     (c)->found_count == occurrence_option)
@@ -641,12 +650,15 @@ bool removed_prefixes_p (void);
 
 void usage (int);
 
-int confirm (const char *, const char *);
-void request_stdin (const char *);
+int confirm (const char *message_action, const char *name);
+void request_stdin (const char *option);
 
 void tar_stat_init (struct tar_stat_info *st);
 void tar_stat_destroy (struct tar_stat_info *st);
 void usage (int) __attribute__ ((noreturn));
+int tar_timespec_cmp (struct timespec a, struct timespec b);
+const char *archive_format_string (enum archive_format fmt);
+const char *subcommand_string (enum subcommand c);
 
 /* Module update.c.  */
 
@@ -656,23 +668,28 @@ void update_archive (void);
 
 /* Module xheader.c.  */
 
-void xheader_decode (struct tar_stat_info *);
-void xheader_decode_global (void);
-void xheader_store (char const *, struct tar_stat_info const *, void *);
-void xheader_read (union block *, size_t);
+void xheader_init (struct xheader *xhdr);
+void xheader_decode (struct tar_stat_info *stat);
+void xheader_decode_global (struct xheader *xhdr);
+void xheader_store (char const *keyword, struct tar_stat_info *st,
+		    void const *data);
+void xheader_read (struct xheader *xhdr, union block *header, size_t size);
 void xheader_write (char type, char *name, struct xheader *xhdr);
-void xheader_write_global (void);
-void xheader_finish (struct xheader *);
-void xheader_destroy (struct xheader *);
+void xheader_write_global (struct xheader *xhdr);
+void xheader_finish (struct xheader *hdr);
+void xheader_destroy (struct xheader *hdr);
 char *xheader_xhdr_name (struct tar_stat_info *st);
 char *xheader_ghdr_name (void);
-void xheader_write (char, char *, struct xheader *);
-void xheader_write_global (void);
 void xheader_set_option (char *string);
+void xheader_string_begin (struct xheader *xhdr);
+void xheader_string_add (struct xheader *xhdr, char const *s);
+bool xheader_string_end (struct xheader *xhdr, char const *keyword);
+bool xheader_keyword_deleted_p (const char *kw);
+char *xheader_format_name (struct tar_stat_info *st, const char *fmt,
+			   size_t n);
 
 /* Module system.c */
 
-void sys_stat_nanoseconds (struct tar_stat_info *);
 void sys_detect_dev_null_output (void);
 void sys_save_archive_dev_ino (void);
 void sys_drain_input_pipe (void);
@@ -687,19 +704,35 @@ pid_t sys_child_open_for_compress (void);
 pid_t sys_child_open_for_uncompress (void);
 size_t sys_write_archive_buffer (void);
 bool sys_get_archive_stat (void);
+int sys_exec_command (char *file_name, int typechar, struct tar_stat_info *st);
+void sys_wait_command (void);
+int sys_exec_info_script (const char **archive_name, int volume_number);
 
 /* Module compare.c */
 void report_difference (struct tar_stat_info *st, const char *message, ...);
 
 /* Module sparse.c */
-bool sparse_file_p (struct tar_stat_info *);
-bool sparse_member_p (struct tar_stat_info *);
-bool sparse_fixup_header (struct tar_stat_info *);
-enum dump_status sparse_dump_file (int, struct tar_stat_info *);
-enum dump_status sparse_extract_file (int, struct tar_stat_info *, off_t *);
-enum dump_status sparse_skip_file (struct tar_stat_info *);
-bool sparse_diff_file (int, struct tar_stat_info *);
+bool sparse_member_p (struct tar_stat_info *st);
+bool sparse_fixup_header (struct tar_stat_info *st);
+enum dump_status sparse_dump_file (int, struct tar_stat_info *st);
+enum dump_status sparse_extract_file (int fd, struct tar_stat_info *st,
+				      off_t *size);
+enum dump_status sparse_skip_file (struct tar_stat_info *st);
+bool sparse_diff_file (int, struct tar_stat_info *st);
 
 /* Module utf8.c */
 bool string_ascii_p (const char *str);
 bool utf8_convert (bool to_utf, char const *input, char **output);
+
+/* Module transform.c */
+typedef enum
+  {
+    xform_regfile,
+    xform_link,
+    xform_symlink
+  } xform_type;
+
+void set_transform_expr (const char *expr);
+bool transform_name (char **pinput);
+bool transform_member_name (char **pinput, xform_type type);
+bool transform_name_fp (char **pinput, char *(*fun)(char *, void *), void *);
